import { DatabaseError } from './protocol';
type Resolution = null | string | Error | DatabaseError;
type Resolver = (resolution: Resolution) => void;
type ResultHandler = (resolve: Resolver) => void;
type Callback<T> = (item: T) => void;
export declare class ResultRowImpl<T> {
    readonly names: string[];
    readonly data: any[];
    private readonly lookup;
    constructor(names: string[], data: any[]);
    [Symbol.iterator](): Iterator<any>;
    get(name: string): any;
    reify(): T;
}
/** The default result type, used if no generic type parameter is specified. */
export type ResultRecord = Record<string, any>;
/**
 * A result row provides access to data for a single row.
 *
 * The generic type parameter is carried over from the query method.
 * @interface
 *
 * To retrieve a column value by name use the {@link get} method; or use {@link reify} to convert
 * the row into an object.
 *
 * The {@link data} attribute provides raw access to the row data, but it's also possible to use the
 * spread operator to destructure into a tuple.
 */
export type ResultRow<T = ResultRecord> = Omit<ResultRowImpl<T>, 'get'> & {
    get<K extends keyof T>(name: K): T[K];
};
export declare class Result<T = ResultRecord> {
    names: string[];
    rows: any[][];
    status: null | string;
    constructor(names: string[], rows: any[][], status: null | string);
    [Symbol.iterator](): Iterator<ResultRow<T>>;
    reify(): T[];
}
export declare class ResultIterator<T = ResultRecord> extends Promise<Result<T>> {
    private container;
    private subscribers;
    private done;
    rows: any[][] | null;
    names: string[] | null;
    constructor(container: any[][], executor: ResultHandler);
    first(): Promise<ResultRow<T> | undefined>;
    one(): Promise<ResultRow<T>>;
    reify(): AsyncIterable<T>;
    notify(done: boolean, status?: (string | DatabaseError | Error)): void;
    [Symbol.asyncIterator](): AsyncIterator<ResultRow<T>>;
}
export type DataHandler<T> = Callback<T | Resolution>;
export type NameHandler = Callback<string[]>;
export declare function makeResult<T>(): {
    iterator: ResultIterator<T>;
    dataHandler: DataHandler<any[] | null>;
    nameHandler: (names: string[]) => void;
};
export {};
